<?php

namespace wcf\system\event\listener;

use wcf\system\cache\builder\MyElistingCacheBuilder;

use wcf\system\exception\SystemException;
use wcf\system\WCF;
use wcf\util\DateUtil;
use wcf\util\JSON;

/** Extends Ebay, Amazon, PlayAsia Links by Affiliate Parameters
 *
 * @author      PCSG (Jan)
 * @copyright   2001-2009 WoltLab GmbH, 2014-2015 Oliver Schlöbe
 * @license        GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package        com.woltlab.wcf
 * @subpackage    system.event.listener
 * @category    Community Framework
 */
class MyelistingLinkReplacerAfterParse implements IParameterizedEventListener
{

    public $parsed = false;

    // Link to the my.eListing API (change in options.xml)
    private $api = 'https://'.MYELISTING_URL.'/api';

    // Is the my.eListing API available?
    private $api_available;

    // Our chache
    private $cache;

    /**
     * @see \wcf\system\event\listener\IParameterizedEventListener::execute()
     */
    public function execute($eventObj, $className, $eventName, array &$parameters)
    {
        // If the link preview is enabled in settings...
        if(MYELISTING_LINK_PREVIEW) {

            // Check if we already tested if API is available
            $this->cache = MyElistingCacheBuilder::getInstance();

            try {
                $this->api_available = $this->cache->getData(array(), 'apiAvailable');
                if(isset($this->api_available[0])) {
                    $this->api_available = JSON::decode($this->api_available[0], true);
                } else {
                    $this->api_available = null;
                }
            } catch(SystemException $e) {
                $this->api_available = null;
            }

            // If we didn't test if API is available do it now and store result in cache
            if ($this->api_available === null) {
                $parameters = array("function" => "apiAvailable", "value" => "x"); // value = x so that value is set
                $data = $this->cache->rebuild($parameters);
                $this->api_available = JSON::decode($data["apiAvailable"][0], true);
            }

            // If the API is available do the parsing for this post
            if($this->api_available) {
                $eventObj->message = $this->makeEbayPreview($eventObj->message);
            }
        }
    }


    /**
     * Turns an eBay URL in an article preview HTML
     *
     * @param String $message
     * @return String $message
     */
    private function makeEbayPreview($message = "")
    {
        $campaignID = '5337961799';
        $customerID = 'EBAY-DE';
        $programID = '707-53477-19255-0';

        preg_match("/http:\/\/[a-z]*\.ebay\.(at|ch|com|de)\//i", $message, $matches);

        // If we have a ebay link in our post
        if (count($matches)) {

            // Load HTML from post
            $dom = new \DOMDocument();
            $dom->encoding = "utf-8";
            $dom->formatOutput = true;
            $dom->strictErrorChecking = false;
            $dom->loadHTML(utf8_decode($message), LIBXML_HTML_NODEFDTD);

            $urls = $dom->getElementsByTagName("a");

            $replacingNodes = array();
            $nodesToReplace = array();


            foreach($urls as $key=>$url) {

                $url->setAttribute('target', '_blank');
                $href = $url->getAttribute("href");
                $orig_link = $href;

                // Change the link to an eBay rover link redirected over my.eListing
                if (count($matches)) {
                    $href = "http://rover.ebay.com/rover/1/$programID/1?ff3=2&toolid=10044&campid=" . urlencode($campaignID) . "&customid=" . urlencode($customerID) . "&mpre=" . $href;
                    $href = $this->api.'/redirect/?to=' . rawurlencode($href);
                    $url->setAttribute("href", $href);
                }

                // Get the article ID from the URL
                $parsedLink = parse_url($orig_link);

                if (array_key_exists('query', $parsedLink) && strpos($parsedLink['query'], 'item=')) {

                    // copy ID from end of 'item=' to first ' " '
                    $itemID = substr($parsedLink['query'], strpos($parsedLink['query'], 'item=') + 5,
                        strpos($parsedLink['query'], '"') - strpos($parsedLink['query'], 'item=') - 5);

                } else {
                    $item = $parsedLink['path'];
                    $itemID = substr($item, strrpos($item, '/') + 1);
                }


                // If we don't get an number as ID continue with next link
                if (!is_numeric($itemID)) {
                    continue;
                }


                // Check if we have this article details already cached
                try {
                    $getItemByID = $this->cache->getData(array(), 'getItemById');
                    if(isset($getItemByID[$itemID])) {
                        $itemInfo = JSON::decode($getItemByID[$itemID], true);
                    } else {
                        $itemInfo = null;
                    }
                } catch(SystemException $e) {
                    $itemInfo = null;
                }

                if ($itemInfo === null) {
                    $parameters = array("function" => "getItemById", "value" => $itemID);
                    $data = $this->cache->rebuild($parameters);
                    $itemInfo = JSON::decode($data["getItemById"][$itemID], true);
                }


                // If getting the details returned an error show an error instead of preview
                if (isset($itemInfo['error'])) {

                    $error = true;
                    $data  = array();

                    if($itemInfo['error'] == 'apiUnavailable') {
                        $msg = WCF::getLanguage()->get('myelisting.api.unavailable');
                    } else {
                        $msg = WCF::getLanguage()->get('myelisting.link.notfound');
                    }

                    // If error cause of unavailable API don't cache the unavailable item
//                    if($itemInfo['error'] != 'apiUnavailable') {
//                        $this->cache->store("getItemById", $itemID, $itemInfo);
//                    }
                } else {

                    // Format/convert the article details

                    $error = false;

                    $date = new \DateTime;
                    $date->setTimestamp( strtotime( $itemInfo['timestamp'] ) );

                    $timestamp = DateUtil::format($date, 'H:i');

                    $time_name = WCF::getLanguage()->get('myelisting.time.name');

                    $thumbnail = $itemInfo['thumbnail'];

                    $title = $itemInfo['title'];
                    $price = money_format('%.2n', floatval($itemInfo['price'])) . ' ' . $itemInfo['currency'];
                    $timeLeft = $this->getPrettyTimeFromEbayTime($itemInfo['timeLeft']);
                    $lastUpdateMsg = WCF::getLanguage()->get('myelisting.link.last_updated');

                    $data = array( 'title'         => $title,
                                   'price'         => $price,
                                   'timeLeft'      => $timeLeft,
                                   'lastUpdateMsg' => $lastUpdateMsg,
                                   'thumbnail'     => $thumbnail      );

                }


                $nodesToReplace[$key] = $url;

                // Variables to pass to the template
                $tplVars = array( 'error'   => $error,
                                  'data'    => $data,
                                  'href'    => $href,
                                  'wcfPath' => WCF::getPath() );

                // If we had an error add the error message to tplVars
                if($error) {
                    $tplVars['msg'] = $msg;
                }

                // Pass vars to preview template and get html back
                $tpl = WCF::getTPL()->fetch('myelistingLinkPreview','wcf',$tplVars);

                // Create DomNode from template HTML and store it to replace it later
                $preview = new \DOMDocument();
                $preview->encoding = "utf-8";
                $preview->formatOutput = true;
                $preview->strictErrorChecking = false;
                $preview->loadHTML(utf8_decode($tpl), LIBXML_HTML_NODEFDTD);

                $preview = $dom->importNode($preview->documentElement, true);

                $replacingNodes[$key] = $preview;


            }

            // Replace all links with their preview
            foreach($nodesToReplace as $key=>$nodeToReplace) {
                $nodeToReplace->parentNode->replaceChild($replacingNodes[$key], $nodeToReplace);
            }

            $message = $dom->saveHTML();

        }

        return $message;

    }


    /**
     * Convert the eBay timestamp to a pretty format
     *
     * @param $eBayTimeString Input is of form 'PT12M25S'
     * @return string
     */
    static function getPrettyTimeFromEbayTime($eBayTimeString)
    {

        if ($eBayTimeString == 'PT0S') {
            return "Auktion beendet.";
        }

        $matchAry = array();
        $pattern = "#P([0-9]{0,3}D)?T([0-9]?[0-9]H)?([0-9]?[0-9]M)?([0-9]?[0-9]S)#msiU";
        preg_match($pattern, $eBayTimeString, $matchAry);

        $days = false;
        $hours = false;
        $min = false;
        $sec = false;

        if (isset($matchAry[1])) {
            $days = (int)$matchAry[1];
        }

        if (isset($matchAry[2])) {
            $hours = (int)$matchAry[2];
        }

        if (isset($matchAry[3])) {
            $min = (int)$matchAry[3];
        }

        if (isset($matchAry[4])) {
            $sec = (int)$matchAry[4];
        }


        $retnStr = '';

        if ($days) {
            $msg = WCF::getLanguage()->get('myelisting.time.days');
            $retnStr .= "$days $msg";
        }

        if ($hours) {
            $msg = WCF::getLanguage()->get('myelisting.time.hours');
            $retnStr .= " $hours $msg";
        }

        if ($min) {
            $msg = WCF::getLanguage()->get('myelisting.time.minutes');
            $retnStr .= " $min $msg";
        }

        if ($sec) {
            $msg = WCF::getLanguage()->get('myelisting.time.seconds');
            $retnStr .= " $sec $msg";
        }

        return $retnStr;
    }

}